import os

ADMIN_ADDRESS = 0x032611AA4C7610D0B6306ADFFA27BF3E8774D1A27C3E9F93CDC2F7BCFEC0C3D7
ORACLE_CONTROLLER_ADDRESS = (
    0x013BEFE6EDA920CE4AF05A50A67BD808D67EEE6BA47BB0892BEF2D630EAF1BBA
)
PUBLISHER_REGISTRY_ADDRESS = (
    0x07E05E4DEA8A62988D9A06EA47BDAC34C759A413DB5B358E4A3A3D691D9D89E4
)
NETWORK = "testnet"
DEFAULT_AGGREGATION_MODE = 0

if os.environ.get("__PONTIS_STAGING_ENV__") == "TRUE":
    print("Warning: Communicating with staging contracts, not production")
    ORACLE_CONTROLLER_ADDRESS = (
        0x0144B11738F284CA4E6E0B58C71CA814BB025DA724D367550E86AD7925CF21EE
    )
    PUBLISHER_REGISTRY_ADDRESS = (
        0x014385A4C5006834E4407C09E08CD3739FA0221D92CE2FEEC9B830D206723C70
    )
